/************************************************
 * This code is free to use, but this copyright
 * should be deleted, otherwise, they won't 
 * think you wrote it!
 * 
 * If you find this useful, please email me at
 * aprenot@aaronprenot.com
 * 
 * (c)2004 - Aaron Prenot
 * http://www.aaronprenot.com
 * **********************************************/

using System;
using System.Collections.Generic;

namespace Diff
{
	/// <summary>
	/// Creates a Matrix of boolean values stored in bytes.
	/// </summary>
	public class BitMatrix
	{
		protected byte[]	_bytes;
		protected KeyValuePair<int,int>	_size;
		public BitMatrix(KeyValuePair<int,int> sz) : this(sz.Key, sz.Value)
		{
			// 
			// TODO: Add constructor logic here
			//
		}

		public BitMatrix(int square) : this(square, square)
		{

		}

		public BitMatrix(int width, int height)
		{
			int numBytes = (width * height);
			if ( (numBytes % 8) > 0 ) numBytes += (8 - (numBytes % 8));
			numBytes /= 8;
			_bytes = new byte[numBytes];
			for ( int i = 0; i < numBytes; i++ ) _bytes[i] = 0;
			_size = new KeyValuePair<int,int>(width, height);

		}

		public bool this [int x, int y]
		{
			get { return GetBit(x,y); }
			set { SetBit(x,y,value); }
		}

		public int Width
		{
			get { return _size.Key; }
		}

		public int Height
		{
			get { return _size.Value; }
		}

		public bool GetBit(int x, int y)
		{
			if ( x >= _size.Key ) throw new ArgumentOutOfRangeException("x");
			if ( y >= _size.Value ) throw new ArgumentOutOfRangeException("y");
			int bit = 0;
			int location = (y * _size.Key);
			location += x;
			bit = location % 8;
			location /= 8;

			byte bitVal = 1;
			bitVal <<= (8 - (bit + 1));
			return ( (_bytes[location] & bitVal) == bitVal );
		}

		public void SetBit(int x, int y, bool value)
		{
			if ( x >= _size.Key ) throw new ArgumentOutOfRangeException("x");
			if ( y >= _size.Value ) throw new ArgumentOutOfRangeException("y");
			int bit = 0;
			int location = (y * _size.Key);
			location += x;
			bit = location % 8;
			location /= 8;

			byte bitVal = 1;
			bitVal <<= (8 - (bit + 1));
			if ( value ) _bytes[location] |= bitVal;
			else 
			{
				if ( (_bytes[location] & bitVal) == bitVal ) _bytes[location] ^= bitVal;	// bit has been set, so clear it
			}
		}

		public void Clear()
		{
			Clear(false);
		}

		public void Clear(bool Value)
		{
			for ( int i = 0; i < _bytes.Length; i++) _bytes[i] = (Value) ? ( byte ) 1 : ( byte ) 0;
		}
	}
}
