﻿using System;
using System.IO;
using System.Linq;
using System.Collections;
using System.Collections.Generic;

class MainClass
{
	public static int levenshtein(string s, string t)
	{
		int n = s.Length;
		int m = t.Length;
		int[,] d = new int[n + 1, m + 1];

		// Step 1
		if (n == 0)
		{
			return m;
		}

		if (m == 0)
		{
			return n;
		}

		// Step 2
		for (int i = 0; i <= n; d[i, 0] = i++)
		{
		}

		for (int j = 0; j <= m; d[0, j] = j++)
		{
		}

		// Step 3
		for (int i = 1; i <= n; i++)
		{
			//Step 4
			for (int j = 1; j <= m; j++)
			{
				// Step 5
				int cost = (t[j - 1] == s[i - 1]) ? 0 : 1;

				// Step 6
				d[i, j] = Math.Min(
					Math.Min(d[i - 1, j] + 1, d[i, j - 1] + 1),
					d[i - 1, j - 1] + cost);
			}
		}
		// Step 7
		return d[n, m];
	}

	public class ComparableStringReader : Diff.ComparableStreamReader
	{
		protected string    _source;
		protected int        _location;
		public ComparableStringReader(string source)
		{
			_source = source;
			_location = 0;
		}

		public override IComparable GetNext()
		{
			if ( _location < _source.Length ) 
				return _source.Substring(_location++, 1);
			else 
				return null;
		}
	}

	public static void Main (string[] args)
	{
		var lines = File.ReadLines ("../list.txt").ToArray ();

		var count = lines.Length;

		int[] rowMin = (new int[count]).Select(i=>100000000).ToArray();

		int[,] metric = new int[count, count];
		for (int i = 0; i < count; i++) {
			for (int j = 0; j < i; j++) {
				var s1 = lines [i];
				var s2 = lines [j];
				var d = levenshtein (s1, s2);
				metric [i, j] = d;
				metric [j, i] = d;
				rowMin [i] = Math.Min (rowMin [i], d);
				rowMin [j] = Math.Min (rowMin [j], d);
			}
		}

		var maxMin = rowMin.Max ();

		List<int[]> edges = new List<int[]> ();

		for (int i = 0; i < count; i++) {
			for (int j = 0; j < i; j++) {
				//if (metric [i, j] <= maxMin) 
				{
					edges.Add ( new int[]{ i, j });
				}
			}
		}

		List<int>[] edgesFrom = new List<int>[count];
		for (int i=0;i<count;i++){
			edgesFrom [i] = new List<int> ();
		}
		foreach (var e in edges) {
			edgesFrom [e [0]].Add (e [1]);
			edgesFrom [e [1]].Add (e [0]);
		}

		for (var i = 0; i < count; i++) {
			edgesFrom [i] = edgesFrom [i].OrderBy (j => metric [i, j]).ToList ();
		}

		//repopulate with top two outgoing edges from each vertex, so we'll always have a choice
		edges.Clear ();

		int minDegree = 3;

		for (var i = 0; i < count; i++) {
			for (var j = 0; j < minDegree; j++) {
				var ea = i;
				var eb = edgesFrom [i][j];
				edges.Add (new int[]{ ea, eb });
				edges.Add (new int[]{ eb, ea });
			}
		}

		List<int>[] graph = new List<int>[count];
		for (var i = 0; i < count; i++) {
			graph [i] = new List<int> ();
		}
		foreach (var e in edges) {
			var i = e [0];
			var j = e [1];
			if (!graph [i].Contains (j)) {
				graph [i].Add (j);
			}
			if (!graph [j].Contains (i)) {
				graph [j].Add (i);
			}
		}

		//check connected
		var visited = new bool[count];
		visited [0] = true;
		bool changed = true;

		while (changed) {
			changed = false;
			for (int i = 0; i < count; i++) {
				if (visited [i]) {
					foreach (var v in graph[i]) {
						if (visited [v] == false) {
							visited [v] = true;
							changed = true;
						}
					}
				}
			}
		}
		if (visited.All(b=>b)==false){
			Console.WriteLine ("EEK GRAPH NOT CONNECTED");
		}

		string[] pages = new string[count];
		for (int i = 0; i < count; i++) {
			string page = "<!DOCTYPE html>\r\n<html lang=\"en\">\r\n  <head>\r\n    <meta charset=\"utf-8\">\r\n    <meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\">\r\n    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">\r\n    <title>Bootstrap 101 Template</title>\r\n\r\n    <!-- Bootstrap -->\r\n    <link href=\"css/bootstrap.min.css\" rel=\"stylesheet\">\r\n<style>\r\na:visited{color:#666 !important;}\r\n\n  body {\n    padding-top: 60px;\n  }</style>\r\n    <!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->\r\n    <!-- WARNING: Respond.js doesn\'t work if you view the page via file:// -->\r\n    <!--[if lt IE 9]>\r\n      <script src=\"https://oss.maxcdn.com/html5shiv/3.7.2/html5shiv.min.js\"></script>\r\n      <script src=\"https://oss.maxcdn.com/respond/1.4.2/respond.min.js\"></script>\r\n    <![endif]-->\r\n  </head>\r\n  <body>\r\n<h1 id=\"title\" class=\"text-center\">";
			page = page + lines [i];
			page = page + "</h1>\r\n</div>\r\n<p>\r\n<hr>\r\n<p  class=\"text-center\">\r\n";
			for (var j = 0; j < graph [i].Count; j++) {
				Diff.DiffEngine de = new Diff.DiffEngine();

				var fromV = i;
				var toV = graph [i] [j];

				var sFrom = lines [fromV];
				var sTo = lines [toV];

				ComparableStringReader csrSource = new ComparableStringReader(sFrom);
				ComparableStringReader csrDest = new ComparableStringReader(sTo);
				Diff.EditScript es = de.Compare(csrSource, csrDest);

				string difftext = "";
				foreach (Diff.DiffCommand d in es) {
					difftext = difftext + d.ToString ();
				}

				string pageref = toV==0?"index":(toV*3575866506U).ToString();

				string line = "<a href=\""+pageref+".html\">"+difftext+"</a><br>\n";

				page = page+ line;
			}
			page = page + "\r\n</p>\r\n</body>\r\n" +
				"</html>";
			pages[i]=page;
		}

		File.WriteAllText ("index.html", pages [0]);
		for (int i=1;i<count;i++){
			File.WriteAllText ((i*3575866506U) + ".html", pages [i]);
		}
	}
}
